# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import copy
from unittest import mock

from neutronclient.common import exceptions as neutron_exception

from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools


class SecurityGroupTestCase(base.ApiTestCase):

    def setUp(self):
        super(SecurityGroupTestCase, self).setUp()

    def test_create_security_group(self):
        self.set_mock_db_items(fakes.DB_VPC_1,
                               fakes.DB_SECURITY_GROUP_1)
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})
        self.db_api.add_item.return_value = fakes.DB_SECURITY_GROUP_2
        self.neutron.create_security_group.return_value = (
            {'security_group': copy.deepcopy(fakes.OS_SECURITY_GROUP_2)})

        resp = self.execute(
            'CreateSecurityGroup',
            {'GroupName': 'groupname',
             'GroupDescription': 'Group description'})
        secgroup_body = (
            {'security_group': {'name': 'groupname',
                                'description': 'Group description'}})
        self.neutron.create_security_group.assert_called_once_with(
            secgroup_body)
        db_group = tools.purge_dict(fakes.DB_SECURITY_GROUP_2, ('id',))
        db_group['vpc_id'] = None
        self.db_api.add_item.assert_called_once_with(mock.ANY, 'sg', db_group)
        self.neutron.create_security_group.reset_mock()
        self.db_api.add_item.reset_mock()

        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})
        resp = self.execute(
            'CreateSecurityGroup',
            {'VpcId': fakes.ID_EC2_VPC_1,
             'GroupName': 'groupname',
             'GroupDescription': 'Group description'})
        self.assertEqual(fakes.ID_EC2_SECURITY_GROUP_2, resp['groupId'])
        self.db_api.add_item.assert_called_once_with(
            mock.ANY, 'sg',
            tools.purge_dict(fakes.DB_SECURITY_GROUP_2, ('id',)))
        self.neutron.create_security_group.assert_called_once_with(
            secgroup_body)
        self.neutron.create_security_group.reset_mock()
        self.db_api.add_item.reset_mock()

        self.configure(disable_ec2_classic=True)
        self.add_mock_db_items(fakes.DB_VPC_DEFAULT,
                               fakes.DB_SECURITY_GROUP_DEFAULT)
        self.neutron.create_security_group.return_value = (
            {'security_group': copy.deepcopy(fakes.OS_SECURITY_GROUP_5)})
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1),
                                 fakes.OS_SECURITY_GROUP_DEFAULT]})
        self.db_api.add_item.return_value = fakes.DB_SECURITY_GROUP_5

        resp = self.execute(
            'CreateSecurityGroup',
            {'GroupName': 'groupname2',
             'GroupDescription': 'Group description'})
        self.assertEqual(fakes.ID_EC2_SECURITY_GROUP_5, resp['groupId'])
        self.db_api.add_item.assert_called_once_with(
            mock.ANY, 'sg',
            tools.purge_dict(fakes.DB_SECURITY_GROUP_5, ('id',)))
        secgroup_body = (
            {'security_group': {'name': 'groupname2',
                                'description': 'Group description'}})
        self.neutron.create_security_group.assert_called_once_with(
            secgroup_body)

    def test_create_security_group_invalid(self):

        def do_check(args, error_code):
            self.neutron.reset_mock()
            self.db_api.reset_mock()
            self.assert_execution_error(
                error_code, 'CreateSecurityGroup', args)

        self.set_mock_db_items()
        do_check({'VpcId': fakes.ID_EC2_VPC_1,
                  'GroupName': 'groupname',
                  'GroupDescription': 'Group description'},
                 'InvalidVpcID.NotFound')
        self.db_api.get_item_by_id.assert_called_once_with(mock.ANY,
                                                           fakes.ID_EC2_VPC_1)

        do_check({'VpcId': fakes.ID_EC2_VPC_1,
                  'GroupName': 'aa #^% -=99',
                  'GroupDescription': 'Group description'},
                 'ValidationError')

        do_check({'VpcId': fakes.ID_EC2_VPC_1,
                  'GroupName': 'groupname',
                  'GroupDescription': 'aa #^% -=99'},
                 'ValidationError')

        do_check({'GroupName': 'aa \t\x01\x02\x7f',
                  'GroupDescription': 'Group description'},
                 'ValidationError')

        do_check({'GroupName': 'groupname',
                  'GroupDescription': 'aa \t\x01\x02\x7f'},
                 'ValidationError')

        do_check({'GroupName': 'x' * 256,
                  'GroupDescription': 'Group description'},
                 'ValidationError')

        do_check({'GroupName': 'groupname',
                  'GroupDescription': 'x' * 256},
                 'ValidationError')

        do_check({'GroupName': 'groupname'},
                 'MissingParameter')

        do_check({'GroupDescription': 'description'},
                 'MissingParameter')

        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_2, fakes.DB_VPC_1)
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [fakes.OS_SECURITY_GROUP_2]})
        do_check({'VpcId': fakes.ID_EC2_VPC_1,
                  'GroupName': fakes.OS_SECURITY_GROUP_2['name'],
                  'GroupDescription': 'description'},
                 'InvalidGroup.Duplicate')

    def test_create_security_group_over_quota(self):
        self.neutron.create_security_group.side_effect = (
            neutron_exception.OverQuotaClient(413))
        self.assert_execution_error(
            'ResourceLimitExceeded', 'CreateSecurityGroup',
            {'VpcId': fakes.ID_EC2_VPC_1,
             'GroupName': 'groupname',
             'GroupDescription': 'Group description'})
        secgroup_body = (
            {'security_group': {'name': 'groupname',
                                'description': 'Group description'}})
        self.neutron.create_security_group.assert_called_once_with(
            secgroup_body)

    @tools.screen_unexpected_exception_logs
    def test_create_security_group_rollback(self):
        self.set_mock_db_items(fakes.DB_VPC_1)
        self.db_api.add_item.side_effect = Exception()
        self.neutron.create_security_group.return_value = (
            {'security_group': copy.deepcopy(fakes.OS_SECURITY_GROUP_1)})
        self.assert_execution_error(
            self.ANY_EXECUTE_ERROR, 'CreateSecurityGroup',
            {'VpcId': fakes.ID_EC2_VPC_1,
             'GroupName': 'groupname',
             'GroupDescription': 'Group description'})
        self.neutron.delete_security_group.assert_called_once_with(
            fakes.ID_OS_SECURITY_GROUP_1)

    def test_delete_security_group(self):
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1)
        resp = self.execute(
            'DeleteSecurityGroup',
            {'GroupId':
             fakes.ID_EC2_SECURITY_GROUP_1})
        self.assertEqual(True, resp['return'])
        self.db_api.get_item_by_id.assert_any_call(
            mock.ANY,
            fakes.ID_EC2_SECURITY_GROUP_1)
        self.db_api.delete_item.assert_called_once_with(
            mock.ANY,
            fakes.ID_EC2_SECURITY_GROUP_1)
        self.neutron.delete_security_group.assert_called_once_with(
            fakes.ID_OS_SECURITY_GROUP_1)

        self.db_api.delete_item.reset_mock()
        self.neutron.delete_security_group.reset_mock()

        self.configure(disable_ec2_classic=True)
        self.add_mock_db_items(fakes.DB_VPC_DEFAULT,
                               fakes.DB_SECURITY_GROUP_DEFAULT,
                               fakes.DB_SECURITY_GROUP_2,
                               fakes.DB_SECURITY_GROUP_6)
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1),
                                 fakes.OS_SECURITY_GROUP_2,
                                 fakes.OS_SECURITY_GROUP_4,
                                 fakes.OS_SECURITY_GROUP_DEFAULT]})
        self.assert_execution_error(
            'InvalidGroup.NotFound', 'DeleteSecurityGroup',
            {'GroupName': 'groupname2'})

        self.db_api.delete_item.reset_mock()
        self.neutron.delete_security_group.reset_mock()

        self.add_mock_db_items(fakes.DB_SECURITY_GROUP_5)
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1),
                                 fakes.OS_SECURITY_GROUP_2,
                                 fakes.OS_SECURITY_GROUP_4,
                                 fakes.OS_SECURITY_GROUP_5,
                                 fakes.OS_SECURITY_GROUP_DEFAULT]})
        resp = self.execute(
            'DeleteSecurityGroup',
            {'GroupName': 'groupname2'})
        self.assertEqual(True, resp['return'])
        self.db_api.get_item_by_id.assert_any_call(
            mock.ANY,
            fakes.ID_EC2_SECURITY_GROUP_5)
        self.db_api.delete_item.assert_called_with(
            mock.ANY,
            fakes.ID_EC2_SECURITY_GROUP_5)
        self.neutron.delete_security_group.assert_called_once_with(
            fakes.ID_OS_SECURITY_GROUP_5)

    # NOTE(Alex) This test is disabled because it checks using non-AWS id.
    @base.skip_not_implemented
    def test_delete_security_group_nova_os_id(self):
        self.nova.security_groups.list.return_value = (
            [fakes.OS_SECURITY_GROUP_1,
             fakes.OS_SECURITY_GROUP_2])
        resp = self.execute(
            'DeleteSecurityGroup',
            {'GroupId':
             fakes.ID_OS_SECURITY_GROUP_2})
        self.assertEqual(True, resp['return'])
        self.nova.security_groups.delete.assert_called_once_with(
            fakes.ID_OS_SECURITY_GROUP_2)

    def test_delete_security_group_invalid(self):
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1, fakes.DB_VPC_1)
        self.neutron.show_security_group.return_value = (
            {'security_group': fakes.OS_SECURITY_GROUP_1})
        self.assert_execution_error(
            'CannotDelete', 'DeleteSecurityGroup',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_1})
        self.assert_execution_error(
            'InvalidGroup.NotFound', 'DeleteSecurityGroup',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_2})
        self.assertEqual(0, self.neutron.delete_port.call_count)
        self.assert_execution_error(
            'InvalidGroup.NotFound', 'DeleteSecurityGroup',
            {'GroupName': 'badname'})
        self.assertEqual(0, self.neutron.delete_port.call_count)
        self.assert_execution_error(
            'MissingParameter', 'DeleteSecurityGroup', {})
        self.assertEqual(0, self.neutron.delete_port.call_count)

    def test_delete_security_group_is_in_use(self):
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1)
        self.neutron.delete_security_group.side_effect = (
            neutron_exception.Conflict())
        self.assert_execution_error(
            'DependencyViolation', 'DeleteSecurityGroup',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_1})
        self.assertEqual(0, self.db_api.delete_item.call_count)

    def test_describe_security_groups(self):
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_SECURITY_GROUP_2,
                               fakes.DB_SECURITY_GROUP_3,
                               fakes.DB_SECURITY_GROUP_4,
                               fakes.DB_SECURITY_GROUP_5,)
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1),
                                 fakes.OS_SECURITY_GROUP_2,
                                 fakes.OS_SECURITY_GROUP_3,
                                 fakes.OS_SECURITY_GROUP_4,
                                 fakes.OS_SECURITY_GROUP_5]})

        resp = self.execute('DescribeSecurityGroups', {})
        self.assertThat(resp['securityGroupInfo'],
                        matchers.ListMatches(
                            [fakes.EC2_SECURITY_GROUP_1,
                             fakes.EC2_SECURITY_GROUP_2,
                             fakes.EC2_SECURITY_GROUP_3,
                             fakes.EC2_SECURITY_GROUP_4,
                             fakes.EC2_SECURITY_GROUP_5],
                            orderless_lists=True))

        resp = self.execute('DescribeSecurityGroups',
                            {'GroupName.1': 'groupname2'})
        self.assertThat(resp['securityGroupInfo'],
                        matchers.ListMatches(
                            [fakes.EC2_SECURITY_GROUP_4],
                            orderless_lists=True))
        self.assertEqual(0, self.db_api.delete_item.call_count)

        self.db_api.get_items_by_ids = tools.CopyingMock(
            return_value=[fakes.DB_SECURITY_GROUP_4])

        resp = self.execute('DescribeSecurityGroups',
                            {'GroupId.1': fakes.ID_EC2_SECURITY_GROUP_4})
        self.assertThat(resp['securityGroupInfo'],
                        matchers.ListMatches(
                            [fakes.EC2_SECURITY_GROUP_4],
                            orderless_lists=True))
        self.db_api.get_items_by_ids.assert_called_once_with(
            mock.ANY, set([fakes.ID_EC2_SECURITY_GROUP_4]))
        self.assertEqual(0, self.db_api.delete_item.call_count)

        self.check_filtering(
            'DescribeSecurityGroups', 'securityGroupInfo',
            [('vpc-id', fakes.ID_EC2_VPC_1),
             ('group-name', fakes.NAME_DEFAULT_OS_SECURITY_GROUP),
             ('group-id', fakes.ID_EC2_SECURITY_GROUP_1),
             ('description', fakes.EC2_SECURITY_GROUP_1['groupDescription']),
             ('ip-permission.protocol', 'tcp'),
             ('ip-permission.to-port', 10),
             ('ip-permission.from-port', 10),
             ('ip-permission.cidr', '192.168.1.0/24'),
             # TODO(andrey-mp): declare this data in fakes
             # ('ip-permission.group-id', fakes.ID_EC2_SECURITY_GROUP_1),
             # ('ip-permission.group-name', 'default'),
             # ('ip-permission.user-id', fakes.ID_OS_PROJECT),
             ('owner-id', fakes.ID_OS_PROJECT)])
        self.check_tag_support(
            'DescribeSecurityGroups', 'securityGroupInfo',
            fakes.ID_EC2_SECURITY_GROUP_4, 'groupId')

        self.configure(disable_ec2_classic=True)
        self.add_mock_db_items(fakes.DB_VPC_DEFAULT,
                               fakes.DB_SECURITY_GROUP_6)
        resp = self.execute('DescribeSecurityGroups',
                            {'GroupName.1': 'groupname2'})
        self.assertThat(resp['securityGroupInfo'],
                        matchers.ListMatches(
                            [fakes.EC2_SECURITY_GROUP_5],
                            orderless_lists=True))

    @mock.patch('ec2api.api.ec2utils.check_and_create_default_vpc')
    def test_describe_security_groups_no_default_vpc(self, check_and_create):
        self.configure(disable_ec2_classic=True)

        def mock_check_and_create(context):
            self.set_mock_db_items(fakes.DB_VPC_DEFAULT,
                                   fakes.DB_SECURITY_GROUP_DEFAULT)
            self.neutron.list_security_groups.return_value = (
                {'security_groups': [fakes.OS_SECURITY_GROUP_DEFAULT]})
        check_and_create.side_effect = mock_check_and_create

        resp = self.execute('DescribeSecurityGroups', {})
        self.assertThat(resp['securityGroupInfo'],
                        matchers.ListMatches([
                            fakes.EC2_SECURITY_GROUP_DEFAULT],
                        orderless_lists=True))

        check_and_create.assert_called_once_with(mock.ANY)

    def test_repair_default_security_group(self):
        self.db_api.add_item.return_value = fakes.DB_SECURITY_GROUP_1
        self.neutron.create_security_group.return_value = (
            {'security_group': copy.deepcopy(fakes.OS_SECURITY_GROUP_1)})
        self.set_mock_db_items(fakes.DB_VPC_1,
                               fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_SECURITY_GROUP_2)
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [fakes.OS_SECURITY_GROUP_2]})

        resp = self.execute('DescribeSecurityGroups', {})
        self.db_api.restore_item.assert_called_once_with(
            mock.ANY, 'sg',
            {'id': fakes.ID_EC2_VPC_1.replace('vpc', 'sg'),
             'os_id': fakes.ID_OS_SECURITY_GROUP_1,
             'vpc_id': fakes.ID_EC2_VPC_1})
        secgroup_body = (
            {'security_group': {'name': fakes.ID_EC2_VPC_1,
                                'description': 'Default VPC security group'}})
        self.neutron.create_security_group.assert_called_once_with(
            secgroup_body)

    def test_authorize_security_group_invalid(self):

        def check_response(error_code, protocol, from_port, to_port, cidr,
                           group_id=fakes.ID_EC2_SECURITY_GROUP_2):
            params = {'IpPermissions.1.FromPort': str(from_port),
                      'IpPermissions.1.ToPort': str(to_port),
                      'IpPermissions.1.IpProtocol': protocol}
            if group_id is not None:
                params['GroupId'] = group_id
            if cidr is not None:
                params['IpPermissions.1.IpRanges.1.CidrIp'] = cidr
            self.assert_execution_error(
                error_code, 'AuthorizeSecurityGroupIngress', params)
            self.neutron.reset_mock()
            self.db_api.reset_mock()

        self.execute(
            'AuthorizeSecurityGroupIngress',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_2,
             'IpPermissions.1.FromPort': '-1',
             'IpPermissions.1.ToPort': '-1',
             'IpPermissions.1.IpProtocol': 'icmp',
             'IpPermissions.1.IpRanges.1.CidrIp': '0.0.0.0/0'})
        # Duplicate rule
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_SECURITY_GROUP_2)
        self.neutron.create_security_group_rule.side_effect = (
            neutron_exception.Conflict)
        check_response('InvalidPermission.Duplicate', 'icmp',
                       -1, -1, '0.0.0.0/0')
        # Over quota
        self.neutron.create_security_group_rule.side_effect = (
            neutron_exception.OverQuotaClient)
        check_response('RulesPerSecurityGroupLimitExceeded', 'icmp', -1, -1,
                       '0.0.0.0/0')
        # Invalid CIDR address
        check_response('InvalidParameterValue', 'tcp', 80, 81, '0.0.0.0/0444')
        # Missing ports
        check_response('InvalidParameterValue', 'tcp', -1, -1, '0.0.0.0/0')
        # from port cannot be greater than to port
        check_response('InvalidParameterValue', 'tcp', 100, 1, '0.0.0.0/0')
        # For tcp, negative values are not allowed
        check_response('InvalidParameterValue', 'tcp', -1, 1, '0.0.0.0/0')
        # For tcp, valid port range 1-65535
        check_response('InvalidParameterValue', 'tcp', 1, 65599, '0.0.0.0/0')
        # Invalid protocol
        check_response('InvalidParameterValue', 'xyz', 1, 14, '0.0.0.0/0')
        # Invalid port
        check_response('InvalidParameterValue', 'tcp', " ", "gg", '0.0.0.0/0')
        # Invalid icmp port
        check_response('InvalidParameterValue', 'icmp', " ", "gg", '0.0.0.0/0')
        # Invalid CIDR Address
        check_response('InvalidParameterValue', 'icmp', -1, -1, '0.0.0.0')
        # Invalid CIDR Address
        check_response('InvalidParameterValue', 'icmp', 5, 10, '0.0.0.0/')
        # Invalid Cidr ports
        check_response('InvalidParameterValue', 'icmp', 1, 256, '0.0.0.0/0')
        # Missing group
        check_response('MissingParameter', 'tcp', 1, 255, '0.0.0.0/0', None)
        # Missing cidr
        check_response('MissingParameter', 'tcp', 1, 255, None)
        # Invalid remote group
        self.assert_execution_error(
            'InvalidGroup.NotFound', 'AuthorizeSecurityGroupIngress',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_2,
             'IpPermissions.1.IpProtocol': 'icmp',
             'IpPermissions.1.Groups.1.GroupName': 'somegroup',
             'IpPermissions.1.Groups.1.UserId': 'i-99999999'})

    def test_authorize_security_group_ingress_ip_ranges(self):
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_SECURITY_GROUP_2)
        self.neutron.create_security_group_rule.return_value = (
            {'security_group_rule': [fakes.OS_SECURITY_GROUP_RULE_1]})
        self.execute(
            'AuthorizeSecurityGroupIngress',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_2,
             'IpPermissions.1.FromPort': '10',
             'IpPermissions.1.ToPort': '10',
             'IpPermissions.1.IpProtocol': 'tcp',
             'IpPermissions.1.IpRanges.1.CidrIp': '192.168.1.0/24'})
        self.neutron.create_security_group_rule.assert_called_once_with(
            {'security_group_rule':
             tools.purge_dict(fakes.OS_SECURITY_GROUP_RULE_1,
                              {'id', 'remote_group_id', 'tenant_id'})})
        # NOTE(Alex): Openstack extension, AWS-incompability
        # IPv6 is not supported by Amazon.
        self.execute(
            'AuthorizeSecurityGroupIngress',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_2,
             'IpPermissions.1.FromPort': '10',
             'IpPermissions.1.ToPort': '10',
             'IpPermissions.1.IpProtocol': 'tcp',
             'IpPermissions.1.IpRanges.1.CidrIp': '::/0'})
        self.neutron.create_security_group_rule.assert_called_with(
            {'security_group_rule':
             tools.patch_dict(
                 fakes.OS_SECURITY_GROUP_RULE_1, {'remote_ip_prefix': '::/0'},
                 {'id', 'remote_group_id', 'tenant_id'})})

        self.configure(disable_ec2_classic=True)
        self.add_mock_db_items(fakes.DB_VPC_DEFAULT,
                               fakes.DB_SECURITY_GROUP_4,
                               fakes.DB_SECURITY_GROUP_5,
                               fakes.DB_SECURITY_GROUP_6)
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [fakes.OS_SECURITY_GROUP_4,
                                 fakes.OS_SECURITY_GROUP_5]})

        self.execute(
            'AuthorizeSecurityGroupIngress',
            {'GroupName': 'groupname2',
             'IpPermissions.1.FromPort': '10',
             'IpPermissions.1.ToPort': '10',
             'IpPermissions.1.IpProtocol': 'tcp',
             'IpPermissions.1.IpRanges.1.CidrIp': '::/0'})
        security_group_rule = {
            'direction': 'ingress',
            'ethertype': 'IPv4',
            'port_range_min': 10,
            'port_range_max': 10,
            'protocol': 'tcp',
            'remote_ip_prefix': '::/0',
            'security_group_id': fakes.ID_OS_SECURITY_GROUP_5}
        self.neutron.create_security_group_rule.assert_called_with(
            {'security_group_rule': security_group_rule})

    def test_authorize_security_group_egress_groups(self):
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_SECURITY_GROUP_2)
        self.neutron.create_security_group_rule.return_value = (
            {'security_group_rule': [fakes.OS_SECURITY_GROUP_RULE_1]})
        self.execute(
            'AuthorizeSecurityGroupEgress',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_2,
             'IpPermissions.1.FromPort': '10',
             'IpPermissions.1.IpProtocol': '100',
             'IpPermissions.1.Groups.1.GroupId':
             fakes.ID_EC2_SECURITY_GROUP_1})
        self.neutron.create_security_group_rule.assert_called_once_with(
            {'security_group_rule':
             tools.purge_dict(fakes.OS_SECURITY_GROUP_RULE_2,
                              {'id', 'remote_ip_prefix', 'tenant_id',
                               'port_range_max'})})

    def test_revoke_security_group_ingress_ip_ranges(self):
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_SECURITY_GROUP_2)
        self.neutron.show_security_group.return_value = {
            'security_group': fakes.OS_SECURITY_GROUP_2}
        self.neutron.delete_security_group_rule.return_value = True
        self.execute(
            'RevokeSecurityGroupIngress',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_2,
             'IpPermissions.1.FromPort': '10',
             'IpPermissions.1.ToPort': '10',
             'IpPermissions.1.IpProtocol': 'tcp',
             'IpPermissions.1.IpRanges.1.CidrIp': '192.168.1.0/24'})
        self.neutron.show_security_group.assert_called_once_with(
            fakes.ID_OS_SECURITY_GROUP_2)
        self.neutron.delete_security_group_rule.assert_called_once_with(
            fakes.OS_SECURITY_GROUP_RULE_1['id'])

    def test_revoke_security_group_egress_groups(self):
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_SECURITY_GROUP_2)
        self.neutron.show_security_group.return_value = {
            'security_group': fakes.OS_SECURITY_GROUP_2}
        self.neutron.delete_security_group_rule.return_value = True
        self.execute(
            'RevokeSecurityGroupEgress',
            {'GroupId': fakes.ID_EC2_SECURITY_GROUP_2,
             'IpPermissions.1.FromPort': '10',
             'IpPermissions.1.IpProtocol': '100',
             'IpPermissions.1.Groups.1.GroupId':
             fakes.ID_EC2_SECURITY_GROUP_1})
        self.neutron.show_security_group.assert_called_once_with(
            fakes.ID_OS_SECURITY_GROUP_2)
        self.neutron.delete_security_group_rule.assert_called_once_with(
            fakes.OS_SECURITY_GROUP_RULE_2['id'])




import os
import sys

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
ROOT = os.path.abspath(os.path.join(BASE_DIR, "..", ".."))

sys.path.insert(0, ROOT)
sys.path.insert(0, BASE_DIR)

# This is required for ReadTheDocs.org, but isn't a bad idea anyway.
os.environ['DJANGO_SETTINGS_MODULE'] = 'openstack_dashboard.settings'

# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.doctest',
              'sphinx.ext.todo',
              'sphinx.ext.viewcode',
              'openstackdocstheme']

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/ec2-api'
openstackdocs_auto_name = False
openstackdocs_bug_project = 'ec2-api'
openstackdocs_bug_tag = ''

# autodoc generation is a bit aggressive and a nuisance when doing heavy
# text edit cycles.
# execute "export SPHINX_DEBUG=1" in your terminal to disable

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'EC2API Service'
copyright = '2015, OpenStack Foundation'

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
# html_theme_path = ["."]
# html_theme = '_theme'
# html_static_path = ['static']
html_theme = 'openstackdocs'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {"sidebar_mode": "toc"}

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % project


# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
        ('index',
         '%s.tex' % project,
         '%s Documentation' % project,
         'OpenStack Foundation', 'manual'),
]



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib
import hmac
import posixpath

import httplib2
from oslo_cache import core as cache_core
from oslo_config import cfg
from oslo_log import log as logging
import six
import six.moves.urllib.parse as urlparse
import webob

from ec2api import context as ec2_context
from ec2api import exception
from ec2api.i18n import _
from ec2api.metadata import api
from ec2api import utils
from ec2api import wsgi

LOG = logging.getLogger(__name__)
CONF = cfg.CONF
CONF.import_opt('use_forwarded_for', 'ec2api.api.auth')

metadata_opts = [
    cfg.StrOpt('nova_metadata_ip',
               default='127.0.0.1',
               help=_("IP address used by Nova metadata server.")),
    cfg.IntOpt('nova_metadata_port',
               default=8775,
               help=_("TCP Port used by Nova metadata server.")),
    cfg.StrOpt('nova_metadata_protocol',
               default='http',
               choices=['http', 'https'],
               help=_("Protocol to access nova metadata, http or https")),
    cfg.BoolOpt('nova_metadata_insecure',
                default=False,
                help=_("Allow to perform insecure SSL (https) requests to "
                       "nova metadata")),
    cfg.StrOpt('auth_ca_cert',
               help=_("Certificate Authority public key (CA cert) "
                      "file for ssl")),
    cfg.StrOpt('nova_client_cert',
               default='',
               help=_("Client certificate for nova metadata api server.")),
    cfg.StrOpt('nova_client_priv_key',
               default='',
               help=_("Private key of client certificate.")),
    cfg.StrOpt('metadata_proxy_shared_secret',
               default='',
               help=_('Shared secret to sign instance-id request'),
               secret=True),
    cfg.IntOpt("cache_expiration",
        default=15,
        min=0,
        help=_('This option is the time (in seconds) to cache metadata. '
               'Increasing this setting should improve response times of the '
               'metadata API when under heavy load. Higher values may '
               'increase memory usage, and result in longer times for host '
               'metadata changes to take effect.'))
]

CONF.register_opts(metadata_opts, group='metadata')
cache_core.configure(CONF)


class MetadataRequestHandler(wsgi.Application):
    """Serve metadata."""

    def __init__(self):
        if not CONF.cache.enabled:
            LOG.warning("Metadata doesn't use cache. "
                        "Configure cache options to use cache.")
        self.cache_region = cache_core.create_region()
        cache_core.configure_cache_region(CONF, self.cache_region)

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        LOG.debug('Request: %s', req)

        path = req.path_info
        if path == '' or path[0] != '/':
            path = '/' + path
        path = posixpath.normpath(path)
        path_tokens = path.split('/')[1:]
        if path_tokens[0] == 'ec2':
            path_tokens = path_tokens[1:]

        if path_tokens == ['']:
            resp = api.get_version_list()
            return self._add_response_data(req.response, resp)

        try:
            requester = self._get_requester(req)
            if path_tokens[0] == 'openstack':
                return self._proxy_request(req, requester)

            resp = self._get_metadata(path_tokens, requester)
            return self._add_response_data(req.response, resp)

        except exception.EC2MetadataNotFound:
            return webob.exc.HTTPNotFound()
        except Exception:
            LOG.exception("Unexpected error.")
            msg = _('An unknown error has occurred. '
                    'Please try your request again.')
            return webob.exc.HTTPInternalServerError(
                explanation=six.text_type(msg))

    def _proxy_request(self, req, requester):
        headers = self._build_proxy_request_headers(requester)
        nova_ip_port = '%s:%s' % (CONF.metadata.nova_metadata_ip,
                                  CONF.metadata.nova_metadata_port)
        url = urlparse.urlunsplit((
            CONF.metadata.nova_metadata_protocol,
            nova_ip_port,
            req.path_info,
            req.query_string,
            ''))

        h = httplib2.Http(
            ca_certs=CONF.metadata.auth_ca_cert,
            disable_ssl_certificate_validation=(
                    CONF.metadata.nova_metadata_insecure)
        )
        if (CONF.metadata.nova_client_cert and
                CONF.metadata.nova_client_priv_key):
            h.add_certificate(CONF.metadata.nova_client_priv_key,
                              CONF.metadata.nova_client_cert,
                              nova_ip_port)
        resp, content = h.request(url, method=req.method, headers=headers,
                                  body=req.body)

        if resp.status == 200:
            LOG.debug(str(resp))
            req.response.content_type = resp['content-type']
            req.response.body = content
            return req.response
        elif resp.status == 403:
            LOG.warning(
                'The remote metadata server responded with Forbidden. This '
                'response usually occurs when shared secrets do not match.'
            )
            return webob.exc.HTTPForbidden()
        elif resp.status == 400:
            return webob.exc.HTTPBadRequest()
        elif resp.status == 404:
            return webob.exc.HTTPNotFound()
        elif resp.status == 409:
            return webob.exc.HTTPConflict()
        elif resp.status == 500:
            msg = _(
                'Remote metadata server experienced an internal server error.'
            )
            LOG.warning(msg)
            return webob.exc.HTTPInternalServerError(
                explanation=six.text_type(msg))
        else:
            raise Exception(_('Unexpected response code: %s') % resp.status)

    def _build_proxy_request_headers(self, requester):
        signature = self._sign_instance_id(requester['os_instance_id'])
        return {
            'X-Forwarded-For': requester['private_ip'],
            'X-Instance-ID': requester['os_instance_id'],
            'X-Tenant-ID': requester['project_id'],
            'X-Instance-ID-Signature': signature,
        }

    def _sign_instance_id(self, instance_id):
        return hmac.new(
            CONF.metadata.metadata_proxy_shared_secret.encode("utf-8"),
            instance_id.encode(),
            hashlib.sha256).hexdigest()

    def _get_requester(self, req):
        if req.headers.get('X-Metadata-Provider'):
            provider_id, remote_ip = self._unpack_nsx_request(req)
            context = ec2_context.get_os_admin_context()
            os_instance_id, project_id = (
                api.get_os_instance_and_project_id_by_provider_id(
                    context, provider_id, remote_ip))
        else:
            os_instance_id, project_id, remote_ip = (
                self._unpack_neutron_request(req))
        return {'os_instance_id': os_instance_id,
                'project_id': project_id,
                'private_ip': remote_ip}

    def _unpack_neutron_request(self, req):
        os_instance_id = req.headers.get('X-Instance-ID')
        project_id = req.headers.get('X-Tenant-ID')
        signature = req.headers.get('X-Instance-ID-Signature')
        remote_ip = req.headers.get('X-Forwarded-For')

        if not remote_ip:
            raise exception.EC2MetadataInvalidAddress()

        if os_instance_id is None:
            msg = _('X-Instance-ID header is missing from request.')
        elif project_id is None:
            msg = _('X-Tenant-ID header is missing from request.')
        elif not isinstance(os_instance_id, six.string_types):
            msg = _('Multiple X-Instance-ID headers found within request.')
        elif not isinstance(project_id, six.string_types):
            msg = _('Multiple X-Tenant-ID headers found within request.')
        else:
            msg = None

        if msg:
            raise webob.exc.HTTPBadRequest(explanation=msg)

        self._validate_signature(signature, os_instance_id, remote_ip)
        return os_instance_id, project_id, remote_ip

    def _unpack_nsx_request(self, req):
        remote_address = req.headers.get('X-Forwarded-For')
        if remote_address is None:
            msg = _('X-Forwarded-For is missing from request.')
            raise webob.exc.HTTPBadRequest(explanation=msg)
        provider_id = req.headers.get('X-Metadata-Provider')
        if provider_id is None:
            msg = _('X-Metadata-Provider is missing from request.')
            raise webob.exc.HTTPBadRequest(explanation=msg)
        remote_ip = remote_address.split(',')[0]

        if CONF.metadata.metadata_proxy_shared_secret:
            signature = req.headers.get('X-Metadata-Provider-Signature')
            self._validate_signature(signature, provider_id, remote_ip)

        return provider_id, remote_ip

    def _validate_signature(self, signature, requester_id, requester_ip):
        expected_signature = hmac.new(
            CONF.metadata.metadata_proxy_shared_secret.encode("utf-8"),
            requester_id.encode(),
            hashlib.sha256).hexdigest()

        if not (signature and
                utils.constant_time_compare(expected_signature, signature)):
            LOG.warning('X-Instance-ID-Signature: %(signature)s does '
                        'not match the expected value: '
                        '%(expected_signature)s for id: '
                        '%(requester_id)s. Request From: '
                        '%(requester_ip)s',
                        {'signature': signature,
                         'expected_signature': expected_signature,
                         'requester_id': requester_id,
                         'requester_ip': requester_ip})

            msg = _('Invalid proxy request signature.')
            raise webob.exc.HTTPForbidden(explanation=msg)

    def _get_metadata(self, path_tokens, requester):
        context = ec2_context.get_os_admin_context()
        # NOTE(ft): substitute project_id for context to instance's one.
        # It's needed for correct describe and auto update DB operations.
        # It doesn't affect operations via OpenStack's clients because
        # these clients use auth_token field only
        context.project_id = requester['project_id']
        return api.get_metadata_item(context, path_tokens,
                                     requester['os_instance_id'],
                                     requester['private_ip'],
                                     self.cache_region)

    def _add_response_data(self, response, data):
        if isinstance(data, six.text_type):
            response.text = data
        else:
            response.body = data
        response.content_type = 'text/plain'
        return response



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutronclient.common import exceptions as neutron_exception
from unittest import mock

from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools


class AddressTestCase(base.ApiTestCase):

    def setUp(self):
        super(AddressTestCase, self).setUp()

    def test_allocate_ec2_classic_address(self):
        self.configure(external_network=fakes.NAME_OS_PUBLIC_NETWORK)
        self.neutron.list_networks.return_value = (
            {'networks': [{'id': fakes.ID_OS_PUBLIC_NETWORK}]})
        self.neutron.create_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})

        resp = self.execute('AllocateAddress', {})
        self.assertEqual(fakes.IP_ADDRESS_1, resp['publicIp'])
        self.assertEqual('standard', resp['domain'])
        self.assertNotIn('allocationId', resp)
        self.assertEqual(0, self.db_api.add_item.call_count)
        self.neutron.create_floatingip.assert_called_once_with(
            {'floatingip': {
                'floating_network_id':
                fakes.ID_OS_PUBLIC_NETWORK}})

    def test_allocate_vpc_address(self):
        self.configure(external_network=fakes.NAME_OS_PUBLIC_NETWORK)
        self.neutron.list_networks.return_value = (
            {'networks': [{'id': fakes.ID_OS_PUBLIC_NETWORK}]})
        self.neutron.create_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})
        self.db_api.add_item.return_value = fakes.DB_ADDRESS_1

        resp = self.execute('AllocateAddress', {'Domain': 'vpc'})

        self.assertEqual(fakes.IP_ADDRESS_1, resp['publicIp'])
        self.assertEqual('vpc', resp['domain'])
        self.assertEqual(fakes.ID_EC2_ADDRESS_1,
                         resp['allocationId'])
        self.db_api.add_item.assert_called_once_with(
            mock.ANY, 'eipalloc',
            tools.purge_dict(fakes.DB_ADDRESS_1,
                             ('id', 'vpc_id')))
        self.neutron.create_floatingip.assert_called_once_with(
            {'floatingip': {
                'floating_network_id':
                fakes.ID_OS_PUBLIC_NETWORK}})
        self.neutron.list_networks.assert_called_once_with(
            **{'router:external': True,
               'name': fakes.NAME_OS_PUBLIC_NETWORK})
        self.db_api.reset_mock()
        self.neutron.create_floatingip.reset_mock()
        self.neutron.list_networks.reset_mock()

        self.configure(disable_ec2_classic=True)
        resp = self.execute('AllocateAddress', {})

        self.assertEqual(fakes.IP_ADDRESS_1, resp['publicIp'])
        self.assertEqual('vpc', resp['domain'])
        self.assertEqual(fakes.ID_EC2_ADDRESS_1,
                         resp['allocationId'])
        self.db_api.add_item.assert_called_once_with(
            mock.ANY, 'eipalloc',
            tools.purge_dict(fakes.DB_ADDRESS_1,
                             ('id', 'vpc_id')))
        self.neutron.create_floatingip.assert_called_once_with(
            {'floatingip': {
                'floating_network_id':
                fakes.ID_OS_PUBLIC_NETWORK}})
        self.neutron.list_networks.assert_called_once_with(
            **{'router:external': True,
               'name': fakes.NAME_OS_PUBLIC_NETWORK})

    def test_allocate_address_invalid_parameters(self):
        self.assert_execution_error('InvalidParameterValue', 'AllocateAddress',
                                    {'Domain': 'fake_domain'})
        self.assertEqual(0, self.db_api.add_item.call_count)
        self.assertEqual(0, self.neutron.create_floatingip.call_count)

    def test_allocate_address_overlimit(self):
        self.configure(external_network=fakes.NAME_OS_PUBLIC_NETWORK)
        self.neutron.list_networks.return_value = (
            {'networks': [{'id': fakes.ID_OS_PUBLIC_NETWORK}]})
        self.neutron.create_floatingip.side_effect = (
            neutron_exception.OverQuotaClient())
        self.assert_execution_error('AddressLimitExceeded', 'AllocateAddress',
                                    {'Domain': 'vpc'})
        self.assert_execution_error('AddressLimitExceeded', 'AllocateAddress',
                                    {})

    @tools.screen_unexpected_exception_logs
    def test_allocate_address_vpc_rollback(self):
        self.configure(external_network=fakes.NAME_OS_PUBLIC_NETWORK)
        self.neutron.list_networks.return_value = (
            {'networks': [{'id': fakes.ID_OS_PUBLIC_NETWORK}]})
        self.neutron.create_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})
        self.db_api.add_item.side_effect = Exception()

        self.assert_execution_error(self.ANY_EXECUTE_ERROR, 'AllocateAddress',
                                    {'Domain': 'vpc'})

        self.neutron.delete_floatingip.assert_called_once_with(
            fakes.ID_OS_FLOATING_IP_1)

    # TODO(andrey-mp): api code has to be fixed
    # There is no add-floating-ip and remove-floating-ip command in
    # python-novaclient. Those command have been removed since 7.0.0
    # version (ocata) and ec2-api has version >9.1.0 since long.
    @base.skip_not_implemented
    def test_associate_address_ec2_classic(self):
        self.set_mock_db_items(fakes.DB_INSTANCE_1)
        self.neutron.list_floatingips.return_value = (
            {'floatingips': [fakes.OS_FLOATING_IP_1,
                             fakes.OS_FLOATING_IP_2]})
        self.nova.servers.add_floating_ip.return_value = True

        resp = self.execute('AssociateAddress',
                            {'PublicIp': fakes.IP_ADDRESS_1,
                             'InstanceId': fakes.ID_EC2_INSTANCE_1})
        self.assertEqual(True, resp['return'])

        self.nova.servers.add_floating_ip.assert_called_once_with(
            fakes.ID_OS_INSTANCE_1,
            fakes.IP_ADDRESS_1)

    def test_associate_address_vpc(self):

        def do_check(params, fixed_ip):
            resp = self.execute('AssociateAddress', params)
            self.assertEqual(True, resp['return'])
            self.assertEqual(fakes.ID_EC2_ASSOCIATION_1, resp['associationId'])

            self.neutron.update_floatingip.assert_called_once_with(
                fakes.ID_OS_FLOATING_IP_1,
                {'floatingip': {'port_id': fakes.ID_OS_PORT_2,
                                'fixed_ip_address': fixed_ip}})
            self.db_api.update_item.assert_called_once_with(
                mock.ANY,
                tools.update_dict(
                    fakes.DB_ADDRESS_1,
                    {'network_interface_id':
                     fakes.ID_EC2_NETWORK_INTERFACE_2,
                     'private_ip_address': fixed_ip}))

            self.neutron.update_floatingip.reset_mock()
            self.db_api.update_item.reset_mock()

        self.set_mock_db_items(
            fakes.DB_ADDRESS_1, fakes.DB_IGW_1, fakes.DB_IGW_2,
            fakes.DB_NETWORK_INTERFACE_1, fakes.DB_NETWORK_INTERFACE_2)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})
        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1,
                  'InstanceId': fakes.ID_EC2_INSTANCE_1},
                 fakes.IP_NETWORK_INTERFACE_2)

        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1,
                  'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2},
                 fakes.IP_NETWORK_INTERFACE_2)

        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1,
                  'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2,
                  'PrivateIpAddress': fakes.IP_NETWORK_INTERFACE_2_EXT_1},
                 fakes.IP_NETWORK_INTERFACE_2_EXT_1)

        assigned_db_address_1 = tools.update_dict(
            fakes.DB_ADDRESS_1,
            {'network_interface_id': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'private_ip_address': fakes.IP_NETWORK_INTERFACE_1})
        self.add_mock_db_items(assigned_db_address_1)
        assigned_floating_ip_1 = tools.update_dict(
            fakes.OS_FLOATING_IP_1,
            {'fixed_port_id': fakes.ID_OS_PORT_1,
             'fixed_ip_address': fakes.IP_NETWORK_INTERFACE_1})
        self.neutron.show_floatingip.return_value = (
            {'floatingip': assigned_floating_ip_1})
        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1,
                  'InstanceId': fakes.ID_EC2_INSTANCE_1,
                  'AllowReassociation': 'True'},
                 fakes.IP_NETWORK_INTERFACE_2)

        self.configure(disable_ec2_classic=True)
        self.set_mock_db_items(
            fakes.DB_VPC_DEFAULT, fakes.DB_ADDRESS_1, fakes.DB_IGW_1,
            fakes.DB_NETWORK_INTERFACE_2)
        do_check({'PublicIp': fakes.IP_ADDRESS_1,
                  'InstanceId': fakes.ID_EC2_INSTANCE_1},
                 fakes.IP_NETWORK_INTERFACE_2)

    def test_associate_address_vpc_idempotent(self):

        def do_check(params):
            resp = self.execute('AssociateAddress', params)
            self.assertEqual(True, resp['return'])
            self.assertEqual(fakes.ID_EC2_ASSOCIATION_2, resp['associationId'])

        self.set_mock_db_items(fakes.DB_ADDRESS_2,
                               fakes.DB_NETWORK_INTERFACE_1,
                               fakes.DB_NETWORK_INTERFACE_2)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_2})

        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_2,
                  'InstanceId': fakes.ID_EC2_INSTANCE_1})

        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_2,
                  'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2})

        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_2,
                  'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2,
                  'PrivateIpAddress': fakes.IP_NETWORK_INTERFACE_2})

    def test_associate_address_invalid_main_parameters(self):

        def do_check(params, error):
            self.assert_execution_error(error, 'AssociateAddress', params)

        do_check({},
                 'MissingParameter')

        do_check({'PublicIp': '0.0.0.0',
                  'AllocationId': 'eipalloc-0'},
                 'InvalidParameterCombination')

        do_check({'PublicIp': '0.0.0.0'},
                 'MissingParameter')

        do_check({'AllocationId': 'eipalloc-0'},
                 'MissingParameter')

    def test_associate_address_invalid_ec2_classic_parameters(self):
        # NOTE(ft): ec2 classic instance vs allocation_id parameter
        self.set_mock_db_items(fakes.DB_INSTANCE_2)
        self.assert_execution_error('InvalidParameterCombination',
                                    'AssociateAddress',
                                    {'AllocationId': 'eipalloc-0',
                                     'InstanceId': fakes.ID_EC2_INSTANCE_2})

        # NOTE(ft): ec2 classic instance vs not existing public IP
        self.neutron.list_floatingips.return_value = {'floatingips': []}
        self.assert_execution_error('AuthFailure', 'AssociateAddress',
                                    {'PublicIp': fakes.IP_ADDRESS_1,
                                     'InstanceId': fakes.ID_EC2_INSTANCE_2})

        # NOTE(ft): ec2 classic instance vs vpc public ip
        self.add_mock_db_items(fakes.DB_ADDRESS_1, fakes.DB_ADDRESS_2)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})
        self.assert_execution_error('AuthFailure', 'AssociateAddress',
                                    {'PublicIp': fakes.IP_ADDRESS_1,
                                     'InstanceId': fakes.ID_EC2_INSTANCE_2})

    def test_associate_address_invalid_vpc_parameters(self):

        def do_check(params, error):
            self.assert_execution_error(error, 'AssociateAddress', params)

        # NOTE(ft): not registered instance id vs vpc address
        self.set_mock_db_items()
        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1,
                  'InstanceId': fakes.ID_EC2_INSTANCE_1},
                 'InvalidInstanceID.NotFound')

        # NOTE(ft): vpc instance vs public ip parmeter
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_2)
        do_check({'PublicIp': '0.0.0.0',
                  'InstanceId': fakes.ID_EC2_INSTANCE_1},
                 'InvalidParameterCombination')

        # NOTE(ft): vpc instance vs not registered vpc address
        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1,
                  'InstanceId': fakes.ID_EC2_INSTANCE_1},
                 'InvalidAllocationID.NotFound')

        # NOTE(ft): not registered network interface id vs vpc address
        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1,
                  'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1},
                 'InvalidNetworkInterfaceID.NotFound')

        # NOTE(ft): vpc instance vs broken vpc address
        self.set_mock_db_items(fakes.DB_ADDRESS_1,
                               fakes.DB_NETWORK_INTERFACE_2)
        self.neutron.show_floatingip.side_effect = neutron_exception.NotFound
        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1,
                  'InstanceId': fakes.ID_EC2_INSTANCE_1},
                 'InvalidAllocationID.NotFound')
        self.neutron.show_floatingip.side_effect = None

        # NOTE(ft): already associated address vs network interface
        self.set_mock_db_items(fakes.DB_ADDRESS_1, fakes.DB_ADDRESS_2,
                               fakes.DB_NETWORK_INTERFACE_1)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_2})
        self.assert_execution_error(
            'Resource.AlreadyAssociated', 'AssociateAddress',
            {'AllocationId': fakes.ID_EC2_ADDRESS_2,
             'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1})

        # NOTE(ft): already associated address vs vpc instance
        self.set_mock_db_items(
            fakes.DB_ADDRESS_2,
            fakes.gen_db_network_interface(
                fakes.ID_EC2_NETWORK_INTERFACE_1,
                fakes.ID_OS_PORT_1,
                fakes.ID_EC2_VPC_1,
                fakes.ID_EC2_SUBNET_1,
                fakes.IP_NETWORK_INTERFACE_1,
                instance_id=fakes.ID_EC2_INSTANCE_1))
        self.assert_execution_error('Resource.AlreadyAssociated',
                                    'AssociateAddress',
                                    {'AllocationId': fakes.ID_EC2_ADDRESS_2,
                                     'InstanceId': fakes.ID_EC2_INSTANCE_1})

        # NOTE(ft): multiple network interfaces in vpc instance
        # w/o network interface selection
        self.add_mock_db_items(fakes.DB_NETWORK_INTERFACE_2)
        self.assert_execution_error('InvalidInstanceID', 'AssociateAddress',
                                    {'AllocationId': fakes.ID_EC2_ADDRESS_1,
                                     'InstanceId': fakes.ID_EC2_INSTANCE_1})

        # NOTE(ft): internet gateway isn't attached to the vpc
        self.set_mock_db_items(fakes.DB_ADDRESS_1,
                               fakes.DB_NETWORK_INTERFACE_2)
        self.assert_execution_error('Gateway.NotAttached', 'AssociateAddress',
                                    {'AllocationId': fakes.ID_EC2_ADDRESS_1,
                                     'InstanceId': fakes.ID_EC2_INSTANCE_1})

        # NOTE(tikitavi): associate to wrong public ip
        self.configure(disable_ec2_classic=True)
        self.set_mock_db_items(
            fakes.DB_VPC_DEFAULT, fakes.DB_IGW_DEFAULT, fakes.DB_ADDRESS_1,
            fakes.DB_INSTANCE_DEFAULT, tools.update_dict(
                fakes.DB_NETWORK_INTERFACE_DEFAULT,
                {'instance_id': fakes.ID_EC2_INSTANCE_DEFAULT}))
        do_check({'PublicIp': '0.0.0.0',
                  'InstanceId': fakes.ID_EC2_INSTANCE_DEFAULT},
                 'AuthFailure')

    @tools.screen_unexpected_exception_logs
    def test_associate_address_vpc_rollback(self):
        self.set_mock_db_items(fakes.DB_ADDRESS_1, fakes.DB_IGW_1,
                               fakes.DB_NETWORK_INTERFACE_1,
                               fakes.DB_NETWORK_INTERFACE_2)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})
        self.neutron.update_floatingip.side_effect = Exception()

        self.assert_execution_error(self.ANY_EXECUTE_ERROR, 'AssociateAddress',
                                    {'AllocationId': fakes.ID_EC2_ADDRESS_1,
                                     'InstanceId': fakes.ID_EC2_INSTANCE_1})

        self.db_api.update_item.assert_any_call(
            mock.ANY, fakes.DB_ADDRESS_1)

    # TODO(andrey-mp): api code has to be fixed
    # There is no add-floating-ip and remove-floating-ip command in
    # python-novaclient. Those command have been removed since 7.0.0
    # version (ocata) and ec2-api has version >9.1.0 since long.
    @base.skip_not_implemented
    def test_dissassociate_address_ec2_classic(self):
        self.set_mock_db_items(fakes.DB_INSTANCE_1)
        self.nova.servers.remove_floating_ip.return_value = True
        self.neutron.list_floatingips.return_value = (
            {'floatingips': [fakes.OS_FLOATING_IP_1,
                             fakes.OS_FLOATING_IP_2]})
        self.neutron.list_ports.return_value = (
            {'ports': [fakes.OS_PORT_1,
                       fakes.OS_PORT_2]})

        resp = self.execute('DisassociateAddress',
                            {'PublicIp': fakes.IP_ADDRESS_2})
        self.assertEqual(True, resp['return'])
        self.nova.servers.remove_floating_ip.assert_called_once_with(
            fakes.ID_OS_INSTANCE_1,
            fakes.IP_ADDRESS_2)

        # NOTE(Alex) Disassociate unassociated address in EC2 classic
        resp = self.execute('DisassociateAddress',
                            {'PublicIp': fakes.IP_ADDRESS_1})
        self.assertEqual(True, resp['return'])
        self.assertEqual(1, self.nova.servers.remove_floating_ip.call_count)

    def test_dissassociate_address_vpc(self):
        self.set_mock_db_items(fakes.DB_ADDRESS_2)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_2})

        resp = self.execute('DisassociateAddress',
                            {'AssociationId': fakes.ID_EC2_ASSOCIATION_2})
        self.assertEqual(True, resp['return'])

        self.neutron.update_floatingip.assert_called_once_with(
            fakes.ID_OS_FLOATING_IP_2,
            {'floatingip': {'port_id': None}})
        self.db_api.update_item.assert_called_once_with(
            mock.ANY,
            tools.purge_dict(fakes.DB_ADDRESS_2, ['network_interface_id',
                                                  'private_ip_address']))
        self.neutron.update_floatingip.reset_mock()
        self.db_api.update_item.reset_mock()

        self.configure(disable_ec2_classic=True)

        resp = self.execute('DisassociateAddress',
                            {'PublicIp': fakes.IP_ADDRESS_2})
        self.assertEqual(True, resp['return'])

        self.neutron.update_floatingip.assert_called_once_with(
            fakes.ID_OS_FLOATING_IP_2,
            {'floatingip': {'port_id': None}})
        self.db_api.update_item.assert_called_once_with(
            mock.ANY,
            tools.purge_dict(fakes.DB_ADDRESS_2, ['network_interface_id',
                                                  'private_ip_address']))

    def test_dissassociate_address_vpc_idempotent(self):
        self.set_mock_db_items(fakes.DB_ADDRESS_1)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})

        resp = self.execute('DisassociateAddress',
                            {'AssociationId': fakes.ID_EC2_ASSOCIATION_1})
        self.assertEqual(True, resp['return'])

        self.assertEqual(0, self.neutron.update_floatingip.call_count)
        self.assertEqual(0, self.db_api.update_item.call_count)

    def test_disassociate_address_invalid_parameters(self):

        def do_check(params, error):
            self.assert_execution_error(error, 'DisassociateAddress', params)

        do_check({},
                 'MissingParameter')

        do_check({'PublicIp': '0.0.0.0',
                  'AssociationId': 'eipassoc-0'},
                 'InvalidParameterCombination')

        # NOTE(ft): EC2 Classic public IP does not exists
        self.set_mock_db_items()
        self.neutron.list_floatingips.return_value = {'floatingips': []}

        self.assert_execution_error('AuthFailure', 'DisassociateAddress',
                                    {'PublicIp': fakes.IP_ADDRESS_2})

        # NOTE(ft): vpc address vs public ip parameter
        self.set_mock_db_items(fakes.DB_ADDRESS_1)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})
        do_check({'PublicIp': fakes.IP_ADDRESS_1},
                 'InvalidParameterValue')

        # NOTE(ft): not registered address
        self.set_mock_db_items()
        do_check({'AssociationId': fakes.ID_EC2_ASSOCIATION_1},
                 'InvalidAssociationID.NotFound')

        # NOTE(ft): registered broken vpc address
        self.set_mock_db_items(fakes.DB_ADDRESS_2)
        self.neutron.show_floatingip.side_effect = neutron_exception.NotFound
        do_check({'AssociationId': fakes.ID_EC2_ASSOCIATION_2},
                 'InvalidAssociationID.NotFound')

        # NOTE(tikitavi): disassociate to wrong public ip
        self.configure(disable_ec2_classic=True)
        self.set_mock_db_items()
        self.assert_execution_error('AuthFailure', 'DisassociateAddress',
                                    {'PublicIp': fakes.IP_ADDRESS_2})

        # NOTE(tikitavi): disassociate to unassociated ip
        self.set_mock_db_items(fakes.DB_ADDRESS_1)
        self.assert_execution_error('InvalidParameterValue',
                                    'DisassociateAddress',
                                    {'PublicIp': fakes.IP_ADDRESS_1})

    @tools.screen_unexpected_exception_logs
    def test_dissassociate_address_vpc_rollback(self):
        self.set_mock_db_items(fakes.DB_ADDRESS_2)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_2})
        self.neutron.update_floatingip.side_effect = Exception()

        self.assert_execution_error(
            self.ANY_EXECUTE_ERROR, 'DisassociateAddress',
            {'AssociationId': fakes.ID_EC2_ASSOCIATION_2})

        self.db_api.update_item.assert_any_call(
            mock.ANY, fakes.DB_ADDRESS_2)

    def test_release_address_ec2_classic(self):
        self.set_mock_db_items()
        self.neutron.delete_floatingip.return_value = True
        self.neutron.list_floatingips.return_value = (
            {'floatingips': [fakes.OS_FLOATING_IP_1,
                             fakes.OS_FLOATING_IP_2]})

        resp = self.execute('ReleaseAddress',
                            {'PublicIp': fakes.IP_ADDRESS_1})
        self.assertEqual(True, resp['return'])

        self.neutron.delete_floatingip.assert_called_once_with(
            fakes.OS_FLOATING_IP_1['id'])

    def test_release_address_vpc(self):
        self.set_mock_db_items(fakes.DB_ADDRESS_1)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})

        resp = self.execute('ReleaseAddress',
                            {'AllocationId': fakes.ID_EC2_ADDRESS_1})
        self.assertEqual(True, resp['return'])

        self.neutron.delete_floatingip.assert_called_once_with(
            fakes.ID_OS_FLOATING_IP_1)
        self.db_api.delete_item.assert_called_once_with(
            mock.ANY, fakes.ID_EC2_ADDRESS_1)

    @mock.patch('ec2api.api.address.AddressEngineNeutron.disassociate_address')
    def test_release_address_default_vpc(self, disassociate_address):
        self.configure(disable_ec2_classic=True)
        self.set_mock_db_items(fakes.DB_VPC_DEFAULT,
                               fakes.DB_ADDRESS_DEFAULT,
                               fakes.DB_NETWORK_INTERFACE_DEFAULT)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_2})

        resp = self.execute('ReleaseAddress',
                            {'AllocationId': fakes.ID_EC2_ADDRESS_DEFAULT})
        self.assertEqual(True, resp['return'])

        disassociate_address.assert_called_once_with(
            mock.ANY, association_id=fakes.ID_EC2_ASSOCIATION_DEFAULT)
        self.neutron.delete_floatingip.assert_called_once_with(
            fakes.ID_OS_FLOATING_IP_2)
        self.db_api.delete_item.assert_called_once_with(
            mock.ANY, fakes.ID_EC2_ADDRESS_DEFAULT)

    def test_release_address_invalid_parameters(self):

        def do_check(params, error):
            self.assert_execution_error(error, 'ReleaseAddress', params)

        do_check({},
                 'MissingParameter')

        do_check({'PublicIp': '0.0.0.0',
                  'AllocationId': 'eipalloc-0'},
                 'InvalidParameterCombination')

        # NOTE(ft): EC2 Classic public IP is not found
        self.neutron.list_floatingips.return_value = {'floatingips': []}
        do_check({'PublicIp': fakes.IP_ADDRESS_1},
                 'AuthFailure')

        # NOTE(ft): vpc address vs public ip parameter
        self.set_mock_db_items(fakes.DB_ADDRESS_1)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})
        do_check({'PublicIp': fakes.IP_ADDRESS_1},
                 'InvalidParameterValue')

        # NOTE(ft): not registered address
        self.set_mock_db_items()
        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1},
                 'InvalidAllocationID.NotFound')

        # NOTE(ft): registered broken vpc address
        self.set_mock_db_items(fakes.DB_ADDRESS_1)
        self.neutron.show_floatingip.side_effect = neutron_exception.NotFound
        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_1},
                 'InvalidAllocationID.NotFound')
        self.neutron.show_floatingip.side_effect = None

        # NOTE(ft): address is in use
        self.set_mock_db_items(fakes.DB_ADDRESS_2)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_2})
        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_2},
                 'InvalidIPAddress.InUse')

        # NOTE(tikitavi): address is in use in not default vpc
        self.configure(disable_ec2_classic=True)
        self.set_mock_db_items(fakes.DB_VPC_DEFAULT,
                               fakes.DB_VPC_1,
                               fakes.DB_ADDRESS_2,
                               fakes.DB_NETWORK_INTERFACE_2)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_2})

        do_check({'AllocationId': fakes.ID_EC2_ADDRESS_2},
                 'InvalidIPAddress.InUse')

    @tools.screen_unexpected_exception_logs
    def test_release_address_vpc_rollback(self):
        self.set_mock_db_items(fakes.DB_ADDRESS_1)
        self.neutron.show_floatingip.return_value = (
            {'floatingip': fakes.OS_FLOATING_IP_1})
        self.neutron.delete_floatingip.side_effect = Exception()

        self.assert_execution_error(self.ANY_EXECUTE_ERROR, 'ReleaseAddress',
                                    {'AllocationId': fakes.ID_EC2_ADDRESS_1})

        self.db_api.restore_item.assert_called_once_with(
            mock.ANY, 'eipalloc', fakes.DB_ADDRESS_1)

    def test_describe_addresses_vpc(self):
        self.neutron.list_floatingips.return_value = (
            {'floatingips': [fakes.OS_FLOATING_IP_1,
                             fakes.OS_FLOATING_IP_2]})
        self.neutron.list_ports.return_value = (
            {'ports': [fakes.OS_PORT_1,
                       fakes.OS_PORT_2]})
        self.set_mock_db_items(
            fakes.DB_ADDRESS_1, fakes.DB_ADDRESS_2, fakes.DB_INSTANCE_1,
            fakes.DB_NETWORK_INTERFACE_1, fakes.DB_NETWORK_INTERFACE_2)

        resp = self.execute('DescribeAddresses', {})
        self.assertThat(resp['addressesSet'],
                        matchers.ListMatches([fakes.EC2_ADDRESS_1,
                                              fakes.EC2_ADDRESS_2]))

        self.db_api.get_items_by_ids = tools.CopyingMock(
            return_value=[fakes.DB_ADDRESS_1])
        resp = self.execute('DescribeAddresses',
                            {'AllocationId.1': fakes.ID_EC2_ADDRESS_1})
        self.assertThat(resp['addressesSet'],
                        matchers.ListMatches([fakes.EC2_ADDRESS_1]))
        self.db_api.get_items_by_ids.assert_called_once_with(
            mock.ANY, set([fakes.ID_EC2_ADDRESS_1]))

        self.check_filtering(
             'DescribeAddresses', 'addressesSet',
             [('allocation-id', fakes.ID_EC2_ADDRESS_1),
              ('association-id', fakes.ID_EC2_ASSOCIATION_2),
              ('domain', 'vpc'),
              ('instance-id', fakes.ID_EC2_INSTANCE_1),
              ('network-interface-id', fakes.ID_EC2_NETWORK_INTERFACE_2),
              ('network-interface-owner-id', fakes.ID_OS_PROJECT),
              ('private-ip-address', fakes.IP_NETWORK_INTERFACE_2),
              ('public-ip', fakes.IP_ADDRESS_2)])

    def test_describe_addresses_ec2_classic(self):
        self.set_mock_db_items(fakes.DB_INSTANCE_1)
        self.neutron.list_floatingips.return_value = (
            {'floatingips': [fakes.OS_FLOATING_IP_1,
                             fakes.OS_FLOATING_IP_2]})
        self.neutron.list_ports.return_value = (
            {'ports': [fakes.OS_PORT_1,
                       fakes.OS_PORT_2]})
        resp = self.execute('DescribeAddresses', {})
        self.assertThat(resp['addressesSet'],
                        matchers.ListMatches([fakes.EC2_ADDRESS_CLASSIC_1,
                                              fakes.EC2_ADDRESS_CLASSIC_2]))
        resp = self.execute('DescribeAddresses', {'PublicIp.1':
                                                  fakes.IP_ADDRESS_2})
        self.assertThat(resp['addressesSet'],
                        matchers.ListMatches([fakes.EC2_ADDRESS_CLASSIC_2]))



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from neutronclient.common import exceptions as neutron_exception
from unittest import mock

from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools


class IgwTestCase(base.ApiTestCase):

    def setUp(self):
        super(IgwTestCase, self).setUp()
        self.DB_IGW_1_DETACHED = fakes.gen_db_igw(fakes.ID_EC2_IGW_1)
        self.DB_IGW_2_ATTACHED = fakes.gen_db_igw(fakes.ID_EC2_IGW_2,
                                                  fakes.ID_EC2_VPC_2)

    def test_create_igw(self):
        self.db_api.add_item.return_value = fakes.DB_IGW_2

        resp = self.execute('CreateInternetGateway', {})

        self.assertIn('internetGateway', resp)
        igw = resp['internetGateway']
        self.assertThat(fakes.EC2_IGW_2, matchers.DictMatches(igw))
        self.db_api.add_item.assert_called_with(
                mock.ANY, 'igw', {})

    def test_attach_igw(self):
        self.configure(external_network=fakes.NAME_OS_PUBLIC_NETWORK)
        self.set_mock_db_items(fakes.DB_IGW_1, fakes.DB_IGW_2, fakes.DB_VPC_2,
                               fakes.DB_VPN_GATEWAY_1, fakes.DB_VPN_GATEWAY_2)
        self.neutron.list_networks.return_value = (
                {'networks': [{'id': fakes.ID_OS_PUBLIC_NETWORK}]})

        def do_check():
            resp = self.execute(
                    'AttachInternetGateway',
                    {'VpcId': fakes.ID_EC2_VPC_2,
                     'InternetGatewayId': fakes.ID_EC2_IGW_2})

            self.assertEqual(True, resp['return'])
            self.db_api.update_item.assert_called_once_with(
                    mock.ANY, self.DB_IGW_2_ATTACHED)

        do_check()
        self.neutron.add_gateway_router.assert_called_once_with(
                fakes.ID_OS_ROUTER_2,
                {'network_id': fakes.ID_OS_PUBLIC_NETWORK})
        self.neutron.list_networks.assert_called_once_with(
                **{'router:external': True,
                   'name': fakes.NAME_OS_PUBLIC_NETWORK})

        # VPN gateway is already attached
        self.db_api.reset_mock()
        self.neutron.reset_mock()
        vgw_2 = tools.update_dict(fakes.DB_VPN_GATEWAY_2,
                                  {'vpc_id': fakes.ID_EC2_VPC_2})
        self.add_mock_db_items(vgw_2)
        do_check()
        self.assertFalse(self.neutron.add_gateway_router.called)

    def test_attach_igw_invalid_parameters(self):
        def do_check(error_code):
            self.assert_execution_error(
                error_code, 'AttachInternetGateway',
                {'VpcId': fakes.ID_EC2_VPC_2,
                 'InternetGatewayId': fakes.ID_EC2_IGW_2})

            self.assertEqual(0, self.neutron.add_gateway_router.call_count)
            self.assertEqual(0, self.db_api.update_item.call_count)

            self.neutron.reset_mock()
            self.db_api.reset_mock()

        self.set_mock_db_items(fakes.DB_VPC_2)
        do_check('InvalidInternetGatewayID.NotFound')

        self.set_mock_db_items(fakes.DB_IGW_2)
        do_check('InvalidVpcID.NotFound')

        self.set_mock_db_items(self.DB_IGW_2_ATTACHED, fakes.DB_VPC_2)
        do_check('Resource.AlreadyAssociated')

        self.set_mock_db_items(
            fakes.DB_IGW_2, fakes.DB_VPC_2,
            fakes.gen_db_igw(fakes.ID_EC2_IGW_1, fakes.ID_EC2_VPC_2))
        do_check('InvalidParameterValue')

    @tools.screen_unexpected_exception_logs
    def test_attach_igw_rollback(self):
        self.configure(external_network=fakes.NAME_OS_PUBLIC_NETWORK)
        self.set_mock_db_items(fakes.DB_IGW_1, fakes.DB_IGW_2, fakes.DB_VPC_2)
        self.neutron.list_networks.return_value = (
                {'networks': [{'id': fakes.ID_OS_PUBLIC_NETWORK}]})
        self.neutron.add_gateway_router.side_effect = Exception()

        self.assert_execution_error(
            self.ANY_EXECUTE_ERROR, 'AttachInternetGateway',
            {'VpcId': fakes.ID_EC2_VPC_2,
             'InternetGatewayId': fakes.ID_EC2_IGW_2})

        self.db_api.update_item.assert_any_call(
                mock.ANY, fakes.DB_IGW_2)

    def test_detach_igw(self):
        self.set_mock_db_items(fakes.DB_IGW_1, fakes.DB_VPC_1)

        def do_check():
            resp = self.execute(
                    'DetachInternetGateway',
                    {'VpcId': fakes.ID_EC2_VPC_1,
                     'InternetGatewayId': fakes.ID_EC2_IGW_1})
            self.assertEqual(True, resp['return'])
            self.db_api.update_item.assert_called_once_with(
                    mock.ANY, self.DB_IGW_1_DETACHED)

        do_check()
        self.neutron.remove_gateway_router.assert_called_once_with(
                fakes.ID_OS_ROUTER_1)

        # VPN gateway is still attached
        self.db_api.reset_mock()
        self.neutron.reset_mock()
        self.add_mock_db_items(fakes.DB_VPN_GATEWAY_1)
        do_check()
        self.assertFalse(self.neutron.remove_gateway_router.called)

    def test_detach_igw_invalid_parameters(self):
        def do_check(error_code):
            self.assert_execution_error(
                error_code, 'DetachInternetGateway',
                {'VpcId': fakes.ID_EC2_VPC_1,
                 'InternetGatewayId': fakes.ID_EC2_IGW_1})

            self.assertEqual(0, self.neutron.remove_gateway_router.call_count)
            self.assertEqual(0, self.db_api.update_item.call_count)

            self.neutron.reset_mock()
            self.db_api.reset_mock()

        self.set_mock_db_items(fakes.DB_VPC_1)
        do_check('InvalidInternetGatewayID.NotFound')

        self.set_mock_db_items(fakes.DB_IGW_1)
        do_check('InvalidVpcID.NotFound')

        self.set_mock_db_items(self.DB_IGW_1_DETACHED, fakes.DB_VPC_1)
        do_check('Gateway.NotAttached')

    def test_detach_igw_no_router(self):
        self.set_mock_db_items(fakes.DB_IGW_1, fakes.DB_VPC_1)
        self.neutron.remove_gateway_router.side_effect = (
                neutron_exception.NotFound)

        resp = self.execute(
                'DetachInternetGateway',
                {'VpcId': fakes.ID_EC2_VPC_1,
                 'InternetGatewayId': fakes.ID_EC2_IGW_1})

        self.assertEqual(True, resp['return'])
        self.neutron.remove_gateway_router.assert_called_once_with(
                fakes.ID_OS_ROUTER_1)

    @tools.screen_unexpected_exception_logs
    def test_detach_igw_rollback(self):
        self.set_mock_db_items(fakes.DB_IGW_1, fakes.DB_VPC_1)
        self.neutron.remove_gateway_router.side_effect = Exception()

        self.assert_execution_error(
            self.ANY_EXECUTE_ERROR, 'DetachInternetGateway',
            {'VpcId': fakes.EC2_VPC_1['vpcId'],
             'InternetGatewayId': fakes.EC2_IGW_1['internetGatewayId']})

        self.db_api.update_item.assert_any_call(
                mock.ANY, fakes.DB_IGW_1)

    def test_delete_igw(self):
        self.set_mock_db_items(fakes.DB_IGW_2)

        resp = self.execute(
                'DeleteInternetGateway',
                {'InternetGatewayId': fakes.ID_EC2_IGW_2})

        self.assertEqual(True, resp['return'])
        self.db_api.get_item_by_id.assert_called_once_with(mock.ANY,
                                                           fakes.ID_EC2_IGW_2)
        self.db_api.delete_item.assert_called_once_with(mock.ANY,
                                                        fakes.ID_EC2_IGW_2)

    def test_delete_igw_invalid_parameters(self):
        def do_check(error_code):
            self.assert_execution_error(
                error_code, 'DeleteInternetGateway',
                {'InternetGatewayId': fakes.ID_EC2_IGW_1})

            self.assertEqual(0, self.db_api.delete_item.call_count)

            self.neutron.reset_mock()
            self.db_api.reset_mock()

        self.set_mock_db_items()
        do_check('InvalidInternetGatewayID.NotFound')

        self.set_mock_db_items(fakes.DB_IGW_1)
        do_check('DependencyViolation')

    def test_describe_igw(self):
        self.set_mock_db_items(fakes.DB_IGW_1, fakes.DB_IGW_2)

        resp = self.execute('DescribeInternetGateways', {})
        self.assertThat(resp['internetGatewaySet'],
                        matchers.ListMatches([fakes.EC2_IGW_1,
                                              fakes.EC2_IGW_2]))

        resp = self.execute('DescribeInternetGateways',
                            {'InternetGatewayId.1': fakes.ID_EC2_IGW_2})
        self.assertThat(resp['internetGatewaySet'],
                        matchers.ListMatches([fakes.EC2_IGW_2]))
        self.db_api.get_items_by_ids.assert_called_once_with(
            mock.ANY, set([fakes.ID_EC2_IGW_2]))

        self.check_filtering(
            'DescribeInternetGateways', 'internetGatewaySet',
            [('internet-gateway-id', fakes.ID_EC2_IGW_2),
             ('attachment.state', 'available'),
             ('attachment.vpc-id', fakes.ID_EC2_VPC_1)])
        self.check_tag_support(
            'DescribeInternetGateways', 'internetGatewaySet',
            fakes.ID_EC2_IGW_2, 'internetGatewayId')

    @mock.patch('ec2api.api.ec2utils.check_and_create_default_vpc')
    def test_describe_internet_gateways_no_default_vpc(self, check_and_create):
        self.configure(disable_ec2_classic=True)

        def mock_check_and_create(context):
            self.set_mock_db_items(fakes.DB_VPC_DEFAULT,
                                   fakes.DB_IGW_DEFAULT)
        check_and_create.side_effect = mock_check_and_create

        resp = self.execute('DescribeInternetGateways', {})
        self.assertEqual(resp['internetGatewaySet'],
                         [fakes.EC2_IGW_DEFAULT])

        check_and_create.assert_called_once_with(mock.ANY)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Generic Node base class for all workers that run on hosts."""

from oslo_concurrency import processutils
from oslo_config import cfg
from oslo_log import log as logging
from oslo_service import service
from oslo_utils import importutils

from ec2api import exception
from ec2api.i18n import _
from ec2api import wsgi

LOG = logging.getLogger(__name__)

service_opts = [
    cfg.StrOpt('ec2api_listen',
               default="0.0.0.0",
               help='The IP address on which the EC2 API will listen.'),
    cfg.IntOpt('ec2api_listen_port',
               default=8788,
               help='The port on which the EC2 API will listen.'),
    cfg.BoolOpt('ec2api_use_ssl',
                default=False,
                help='Enable ssl connections or not for EC2 API'),
    cfg.IntOpt('ec2api_workers',
               help='Number of workers for EC2 API service. The default will '
                    'be equal to the number of CPUs available.'),
    cfg.StrOpt('metadata_listen',
               default="0.0.0.0",
               help='The IP address on which the metadata API will listen.'),
    cfg.IntOpt('metadata_listen_port',
               default=8789,
               help='The port on which the metadata API will listen.'),
    cfg.BoolOpt('metadata_use_ssl',
                default=False,
                help='Enable ssl connections or not for EC2 API Metadata'),
    cfg.IntOpt('metadata_workers',
               help='Number of workers for metadata service. The default will '
                    'be the number of CPUs available.'),
]

CONF = cfg.CONF
CONF.register_opts(service_opts)


class WSGIService(service.ServiceBase):
    """Provides ability to launch API from a 'paste' configuration."""

    def __init__(self, name, loader=None, max_url_len=None):
        """Initialize, but do not start the WSGI server.

        :param name: The name of the WSGI server given to the loader.
        :param loader: Loads the WSGI application using the given name.
        :returns: None

        """
        self.name = name
        self.manager = self._get_manager()
        self.loader = loader or wsgi.Loader()
        self.app = self.loader.load_app(name)
        self.host = getattr(CONF, '%s_listen' % name, "0.0.0.0")
        self.port = getattr(CONF, '%s_listen_port' % name, 0)
        self.use_ssl = getattr(CONF, '%s_use_ssl' % name, False)
        self.workers = (getattr(CONF, '%s_workers' % name, None) or
                        processutils.get_worker_count())
        if self.workers and self.workers < 1:
            worker_name = '%s_workers' % name
            msg = (_("%(worker_name)s value of %(workers)s is invalid, "
                     "must be greater than 0") %
                   {'worker_name': worker_name,
                    'workers': str(self.workers)})
            raise exception.InvalidInput(msg)
        self.server = wsgi.Server(name,
                                  self.app,
                                  host=self.host,
                                  port=self.port,
                                  use_ssl=self.use_ssl,
                                  max_url_len=max_url_len)
        # Pull back actual port used
        self.port = self.server.port

    def reset(self):
        """Reset server greenpool size to default.

        :returns: None

        """
        self.server.reset()

    def _get_manager(self):
        """Initialize a Manager object appropriate for this service.

        Use the service name to look up a Manager subclass from the
        configuration and initialize an instance. If no class name
        is configured, just return None.

        :returns: a Manager instance, or None.

        """
        fl = '%s_manager' % self.name
        if fl not in CONF:
            return None

        manager_class_name = CONF.get(fl, None)
        if not manager_class_name:
            return None

        manager_class = importutils.import_class(manager_class_name)
        return manager_class()

    def start(self):
        """Start serving this service using loaded configuration.

        Also, retrieve updated port number in case '0' was passed in, which
        indicates a random port should be used.

        :returns: None

        """
        if self.manager:
            self.manager.init_host()
            self.manager.pre_start_hook()
        self.server.start()
        if self.manager:
            self.manager.post_start_hook()

    def stop(self):
        """Stop serving this API.

        :returns: None

        """
        self.server.stop()

    def wait(self):
        """Wait for the service to stop serving this API.

        :returns: None

        """
        self.server.wait()


# NOTE(vish): the global launcher is to maintain the existing
#             functionality of calling service.serve +
#             service.wait
_launcher = None


def serve(server, workers=None):
    global _launcher
    if _launcher:
        raise RuntimeError(_('serve() can only be called once'))

    _launcher = service.launch(CONF, server, workers=workers)


def wait():
    _launcher.wait()



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import netaddr
from neutronclient.common import exceptions as neutron_exception
from oslo_config import cfg
from oslo_log import log as logging

from ec2api.api import common
from ec2api.api import ec2utils
from ec2api.api import network_interface as network_interface_api
from ec2api.api import route_table as route_table_api
from ec2api.api import vpn_gateway as vpn_gateway_api
from ec2api import clients
from ec2api.db import api as db_api
from ec2api import exception
from ec2api.i18n import _


CONF = cfg.CONF
LOG = logging.getLogger(__name__)


"""Subnet related API implementation
"""


Validator = common.Validator


def create_subnet(context, vpc_id, cidr_block,
                  availability_zone=None):
    vpc = ec2utils.get_db_item(context, vpc_id)
    vpc_ipnet = netaddr.IPNetwork(vpc['cidr_block'])
    subnet_ipnet = netaddr.IPNetwork(cidr_block)
    if subnet_ipnet not in vpc_ipnet:
        raise exception.InvalidSubnetRange(cidr_block=cidr_block)

    main_route_table = db_api.get_item_by_id(context, vpc['route_table_id'])
    (host_routes,
     gateway_ip) = route_table_api._get_subnet_host_routes_and_gateway_ip(
            context, main_route_table, cidr_block)
    neutron = clients.neutron(context)
    with common.OnCrashCleaner() as cleaner:
        # NOTE(andrey-mp): set fake name to filter networks in instance api
        os_network_body = {'network': {'name': 'subnet-0'}}
        try:
            os_network = neutron.create_network(os_network_body)['network']
            cleaner.addCleanup(neutron.delete_network, os_network['id'])
            # NOTE(Alex): AWS takes 4 first addresses (.1 - .4) but for
            # OpenStack we decided not to support this as compatibility.
            os_subnet_body = {'subnet': {'network_id': os_network['id'],
                                         'ip_version': '4',
                                         'cidr': cidr_block,
                                         'host_routes': host_routes}}
            os_subnet = neutron.create_subnet(os_subnet_body)['subnet']
            cleaner.addCleanup(neutron.delete_subnet, os_subnet['id'])
        except neutron_exception.OverQuotaClient:
            raise exception.SubnetLimitExceeded()
        try:
            neutron.add_interface_router(vpc['os_id'],
                                         {'subnet_id': os_subnet['id']})
        except neutron_exception.BadRequest:
            raise exception.InvalidSubnetConflict(cidr_block=cidr_block)
        cleaner.addCleanup(neutron.remove_interface_router,
                           vpc['os_id'], {'subnet_id': os_subnet['id']})
        subnet = db_api.add_item(context, 'subnet',
                                 {'os_id': os_subnet['id'],
                                  'vpc_id': vpc['id']})
        cleaner.addCleanup(db_api.delete_item, context, subnet['id'])
        vpn_gateway_api._start_vpn_in_subnet(context, neutron, cleaner,
                                             subnet, vpc, main_route_table)
        neutron.update_network(os_network['id'],
                               {'network': {'name': subnet['id']}})
        # NOTE(ft): In some cases we need gateway_ip to be None (see
        # _get_subnet_host_routes_and_gateway_ip). It's not set during subnet
        # creation to allow automatic configuration of the default port by
        # which subnet is attached to the router.
        neutron.update_subnet(os_subnet['id'],
                              {'subnet': {'name': subnet['id'],
                                          'gateway_ip': gateway_ip}})
    os_ports = neutron.list_ports(tenant_id=context.project_id)['ports']
    return {'subnet': _format_subnet(context, subnet, os_subnet,
                                     os_network, os_ports)}


def delete_subnet(context, subnet_id):
    subnet = ec2utils.get_db_item(context, subnet_id)
    vpc = db_api.get_item_by_id(context, subnet['vpc_id'])
    network_interfaces = network_interface_api.describe_network_interfaces(
        context,
        filter=[{'name': 'subnet-id',
                 'value': [subnet_id]}])['networkInterfaceSet']
    if network_interfaces:
        msg = _("The subnet '%(subnet_id)s' has dependencies and "
                "cannot be deleted.") % {'subnet_id': subnet_id}
        raise exception.DependencyViolation(msg)
    neutron = clients.neutron(context)
    with common.OnCrashCleaner() as cleaner:
        db_api.delete_item(context, subnet['id'])
        cleaner.addCleanup(db_api.restore_item, context, 'subnet', subnet)
        vpn_gateway_api._stop_vpn_in_subnet(context, neutron, cleaner, subnet)
        try:
            neutron.remove_interface_router(vpc['os_id'],
                                            {'subnet_id': subnet['os_id']})
        except neutron_exception.NotFound:
            pass
        cleaner.addCleanup(neutron.add_interface_router,
                           vpc['os_id'],
                           {'subnet_id': subnet['os_id']})
        try:
            os_subnet = neutron.show_subnet(subnet['os_id'])['subnet']
        except neutron_exception.NotFound:
            pass
        else:
            try:
                neutron.delete_network(os_subnet['network_id'])
            except neutron_exception.NetworkInUseClient as ex:
                LOG.warning('Failed to delete network %(os_id)s during '
                            'deleting Subnet %(id)s. Reason: %(reason)s',
                            {'id': subnet['id'],
                             'os_id': os_subnet['network_id'],
                             'reason': ex.message})

    return True


class SubnetDescriber(common.TaggableItemsDescriber):

    KIND = 'subnet'
    FILTER_MAP = {'available-ip-address-count': 'availableIpAddressCount',
                  'cidr': 'cidrBlock',
                  'cidrBlock': 'cidrBlock',
                  'cidr-block': 'cidrBlock',
                  'subnet-id': 'subnetId',
                  'state': 'state',
                  'vpc-id': 'vpcId'}

    def format(self, subnet, os_subnet):
        if not subnet:
            return None
        os_network = next((n for n in self.os_networks
                           if n['id'] == os_subnet['network_id']),
                          None)
        if not os_network:
            self.delete_obsolete_item(subnet)
            return None
        return _format_subnet(self.context, subnet, os_subnet, os_network,
                              self.os_ports)

    def get_name(self, os_item):
        return ''

    def get_os_items(self):
        neutron = clients.neutron(self.context)
        self.os_networks = neutron.list_networks(
            tenant_id=self.context.project_id)['networks']
        self.os_ports = neutron.list_ports(
            tenant_id=self.context.project_id)['ports']
        return neutron.list_subnets(
            tenant_id=self.context.project_id)['subnets']


def describe_subnets(context, subnet_id=None, filter=None):
    ec2utils.check_and_create_default_vpc(context)
    formatted_subnets = SubnetDescriber().describe(context, ids=subnet_id,
                                                   filter=filter)
    return {'subnetSet': formatted_subnets}


def _format_subnet(context, subnet, os_subnet, os_network, os_ports):
    status_map = {'ACTIVE': 'available',
                  'BUILD': 'pending',
                  'DOWN': 'available',
                  'ERROR': 'available'}
    cidr_range = int(os_subnet['cidr'].split('/')[1])
    # NOTE(Alex) First and last IP addresses are system ones.
    ip_count = pow(2, 32 - cidr_range) - 2
    # TODO(Alex): Probably performance-killer. Will have to optimize.
    dhcp_port_accounted = False
    for port in os_ports:
        for fixed_ip in port.get('fixed_ips', []):
            if fixed_ip['subnet_id'] == os_subnet['id']:
                ip_count -= 1
                if port['device_owner'] == 'network:dhcp':
                    dhcp_port_accounted = True
    if not dhcp_port_accounted:
        ip_count -= 1
    return {
        'subnetId': subnet['id'],
        'state': status_map.get(os_network['status'], 'available'),
        'vpcId': subnet['vpc_id'],
        'cidrBlock': os_subnet['cidr'],
        'defaultForAz': 'false',
        'mapPublicIpOnLaunch': 'false',
        'availableIpAddressCount': ip_count
    }



# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from lxml import etree
from unittest import mock

from oslo_config import cfg
from oslo_context import context
from oslo_serialization import jsonutils
from oslotest import base as test_base
import requests
import webob.dec
import webob.exc

from ec2api import api as ec2
from ec2api import exception
from ec2api.tests.unit import tools
from ec2api import wsgi

CONF = cfg.CONF


@webob.dec.wsgify
def conditional_forbid(req):
    """Helper wsgi app returns 403 if param 'die' is 1."""
    if 'die' in req.params and req.params['die'] == '1':
        raise webob.exc.HTTPForbidden()
    return 'OK'


class ExecutorTestCase(test_base.BaseTestCase):
    def setUp(self):
        super(ExecutorTestCase, self).setUp()
        self.executor = ec2.Executor()

    def _execute(self, invoke):
        class Fake(object):
            pass
        fake_ec2_request = Fake()
        fake_ec2_request.invoke = invoke

        fake_wsgi_request = Fake()

        fake_wsgi_request.environ = {
            'ec2api.context': mock.Mock(
                request_id=context.generate_request_id()),
            'ec2.request': fake_ec2_request,
        }
        return self.executor(fake_wsgi_request)

    def _extract_message(self, result):
        tree = etree.fromstring(result.body)
        return tree.findall('./Errors')[0].find('Error/Message').text

    def _extract_code(self, result):
        tree = etree.fromstring(result.body)
        return tree.findall('./Errors')[0].find('Error/Code').text

    def test_instance_not_found(self):
        def not_found(context):
            raise exception.InvalidInstanceIDNotFound(id='i-01')
        result = self._execute(not_found)
        self.assertIn('i-01', self._extract_message(result))
        self.assertEqual('InvalidInstanceID.NotFound',
                         self._extract_code(result))

    def test_instance_not_found_none(self):
        def not_found(context):
            raise exception.InvalidInstanceIDNotFound(id=None)

        # NOTE(mikal): we want no exception to be raised here, which was what
        # was happening in bug/1080406
        result = self._execute(not_found)
        self.assertIn('None', self._extract_message(result))
        self.assertEqual('InvalidInstanceID.NotFound',
                         self._extract_code(result))

    def test_snapshot_not_found(self):
        def not_found(context):
            raise exception.InvalidSnapshotNotFound(id='snap-01')
        result = self._execute(not_found)
        self.assertIn('snap-01', self._extract_message(result))
        self.assertEqual('InvalidSnapshot.NotFound',
                         self._extract_code(result))

    def test_volume_not_found(self):
        def not_found(context):
            raise exception.InvalidVolumeNotFound(id='vol-01')
        result = self._execute(not_found)
        self.assertIn('vol-01', self._extract_message(result))
        self.assertEqual('InvalidVolume.NotFound', self._extract_code(result))


class FakeResponse(object):
    reason = "Test Reason"

    def __init__(self, status_code=400):
        self.status_code = status_code

    def json(self):
        return {}


class KeystoneAuthTestCase(test_base.BaseTestCase):
    def setUp(self):
        super(KeystoneAuthTestCase, self).setUp()
        self.kauth = ec2.EC2KeystoneAuth(conditional_forbid)

    def _validate_ec2_error(self, response, http_status, ec2_code):
        self.assertEqual(response.status_code, http_status,
                         'Expected HTTP status %s' % http_status)
        root_e = etree.XML(response.body)
        self.assertEqual(root_e.tag, 'Response',
                         "Top element must be Response.")
        errors_e = root_e.find('Errors')
        error_e = errors_e[0]
        code_e = error_e.find('Code')
        self.assertIsNotNone(code_e, "Code element must be present.")
        self.assertEqual(code_e.text, ec2_code)

    def test_no_signature(self):
        req = wsgi.Request.blank('/test')
        resp = self.kauth(req)
        self._validate_ec2_error(resp, 400, 'AuthFailure')

    def test_no_key_id(self):
        req = wsgi.Request.blank('/test')
        req.GET['Signature'] = 'test-signature'
        resp = self.kauth(req)
        self._validate_ec2_error(resp, 400, 'AuthFailure')

    @mock.patch.object(requests, 'request', return_value=FakeResponse())
    def test_communication_failure(self, mock_request):
        req = wsgi.Request.blank('/test')
        req.GET['Signature'] = 'test-signature'
        req.GET['AWSAccessKeyId'] = 'test-key-id'
        resp = self.kauth(req)
        self._validate_ec2_error(resp, 400, 'AuthFailure')
        mock_request.assert_called_with('POST',
                                        CONF.keystone_ec2_tokens_url,
                                        data=mock.ANY, headers=mock.ANY)

    @tools.screen_all_logs
    @mock.patch.object(requests, 'request', return_value=FakeResponse(200))
    def test_no_result_data(self, mock_request):
        req = wsgi.Request.blank('/test')
        req.GET['Signature'] = 'test-signature'
        req.GET['AWSAccessKeyId'] = 'test-key-id'
        resp = self.kauth(req)
        self._validate_ec2_error(resp, 400, 'AuthFailure')
        mock_request.assert_called_with('POST',
                                        CONF.keystone_ec2_tokens_url,
                                        data=mock.ANY, headers=mock.ANY)

        fake_request = mock.NonCallableMock(status_code=200, headers={})
        fake_request.json.return_value = {'token': {}}
        mock_request.return_value = fake_request
        resp = self.kauth(req)
        self._validate_ec2_error(resp, 400, 'AuthFailure')

        fake_request.json.return_value = {'access': {}}
        resp = self.kauth(req)
        self._validate_ec2_error(resp, 400, 'AuthFailure')

    @tools.screen_unexpected_exception_logs
    @mock.patch.object(requests, 'request', return_value=FakeResponse(200))
    def test_params_for_keystone_call(self, mock_request):
        req = wsgi.Request.blank('/test')
        req.GET['Signature'] = 'test-signature'
        req.GET['AWSAccessKeyId'] = 'test-key-id'
        self.kauth(req)
        mock_request.assert_called_with(
            'POST', CONF.keystone_ec2_tokens_url,
            data=mock.ANY, headers=mock.ANY)

        data = jsonutils.loads(mock_request.call_args[1]['data'])
        expected_data = {
            'ec2Credentials': {
                'access': 'test-key-id',
                'headers': {'Host': 'localhost:80'},
                'host': 'localhost:80',
                'verb': 'GET',
                'params': {'AWSAccessKeyId': 'test-key-id'},
                'signature': 'test-signature',
                'path': '/test',
                'body_hash': 'e3b0c44298fc1c149afbf4c8996fb924'
                             '27ae41e4649b934ca495991b7852b855'}}
        self.assertEqual(expected_data, data)
